exec("./Support_Server.cs");

if(isFile("Add-Ons/System_ReturnToBlockland/server.cs"))
{
	// Makes sure the thing is running.
	if(!$RTB::RTBR_ServerControl_Hook)
	{
		exec("Add-Ons/System_ReturnToBlockland/RTBR_ServerControl_Hook.cs");
	}
	
	// Description, ModName, Variable, Properties, Mod, Default value, Requires Restart, Super-Admin Only, Callback Function
	RTB_registerPref("Enable Health Bar", "Health Settings", "$HealthBar::Enabled", "bool", "Script_Healthbar", true, false, false, "Healthbar_UpdateMethod");
	RTB_registerPref("Minigames Only?", "Health Settings", "$HealthBar::MiniOnly", "bool", "Script_Healthbar", true, false, false, "Healthbar_UpdateMinigame");
	RTB_registerPref("Mode", "Health Settings", "$HealthBar::Mode", "list Percentage 0 Amount 1", "Script_Healthbar", 0, false, false, "Healthbar_UpdateMethod");
	RTB_registerPref("Show Vehicle Bar?", "Health Settings", "$HealthBar::Vehicles", "bool", "Script_Healthbar", true, false, false, "Healthbar_UpdateMethod");
}

if ($HealthBar::Enabled $= "")
{
	$HealthBar::Enabled = true;
	$HealthBar::MiniOnly = true;
	$HealthBar::Mode = 0;
	$HealthBar::Vehicles = 1;
}

//Updates all player and vehicle healthbars in the server.
function Healthbar_UpdateMethod()
{	
	for(%a = 0; %a < ClientGroup.getCount(); %a++)
	{
		%client = ClientGroup.getObject(%a);

		%client.sendHealthData();
		%client.sendVehicleHealthData();
	}
}

//This gives the client a tracer when it registers the mod with the server.
//It also makes the healthbar to appear if the conditions are met.
function serverCmdRegisterHealthBarMod(%client)
{
	%client.hasHealthBar = true;
	%client.sendHealthData();
}

function serverCmdRegisterHealthBarVehicleMod(%client) //vehicle stuff is separate for compatibility with Iban and Visolator's versions
{
	%client.hasHealthBarVehicle = true;
	%client.sendVehicleHealthData();
}

package Healthbar_Server
{
	function Armor::onDisabled(%this, %obj)
	{
		%obj.sendHealthData();

		return Parent::onDisabled(%this, %obj);
	}
	
	function Armor::onNewDatablock(%this, %obj, %b)
	{
		Parent::onNewDatablock(%this, %obj, %b);

		%obj.sendHealthData();
	}

	//onDamage also handles negative integers.
	function Armor::onDamage(%this, %obj, %damage)
	{
		parent::onDamage(%this, %obj, %damage);

		%obj.sendHealthData();
	}

	function GameConnection::spawnPlayer(%client)
	{
		%result = parent::spawnPlayer(%client);
		
		%client.sendHealthData();
		
		return %result;
	}
	
	function Player::doOverhealDecay(%player)
	{
		parent::doOverhealDecay(%player);

		%player.sendHealthData();
	}

	function ShapeBase::Damage(%obj, %proj, %pos, %damage, %damageType)
	{
		parent::Damage(%obj, %proj, %pos, %damage, %damageType);

		%obj.sendVehicleHealthDataAll();
	}

	function ShapeBase::applyDamage(%obj, %num)
	{
		parent::applyDamage(%obj, %num);

		%obj.sendVehicleHealthDataAll();
	}

	function ShapeBase::setDamageLevel(%obj, %damage)
	{
		parent::setDamageLevel(%obj, %damage);

		%obj.sendVehicleHealthDataAll();
	}

	function ShapeBase::applyRepair(%obj, %num)
	{
		parent::applyRepair(%obj, %num);

		%obj.sendVehicleHealthDataAll();
	}

	function ShapeBase::setRepairRate(%obj, %num)
	{
		parent::setRepairRate(%obj, %num);

		%obj.sendVehicleHealthDataAll();
	}

	function Armor::onMount(%this, %obj, %col, %slot)
	{
		parent::onMount(%this, %obj, %col, %slot);

		%obj.sendVehicleHealthData();
	}

	function Armor::onUnMount(%this, %obj, %col, %slot)
	{
		parent::onUnMount(%this, %obj, %col, %slot);

		%obj.sendVehicleHealthData();
	}
	
	function GameConnection::onClientEnterGame(%client)
	{
		parent::onClientEnterGame(%client);

		commandToClient(%client, 'detectHealthBarMod');
		commandToClient(%client, 'detectHealthBarVehicleMod');
	}
	
};
activatePackage("Healthbar_Server");

function GameConnection::sendHealthData(%client, %val)
{
	if(isObject(%player = %client.player))
		%player.sendHealthData(%val);
}

function Player::sendHealthData(%player, %val)
{
	if(!isObject(%client = %player.client) || !%client.hasHealthBar)
		return;

	if($HealthBar::MiniOnly && !isObject(%client.minigame))
	{
		%player.healthBarVisible = false;

		commandToClient(%client, 'setDamageLevel', 0, 0);
		commandToClient(%client, 'setHealthBarVisible', false);

		return;
	}

	if(!$HealthBar::Enabled || %player.getState() $= "dead")
	{
		%player.healthBarVisible = false;

		commandToClient(%client, 'setDamageLevel', 0, 0);
		commandToClient(%client, 'setHealthBarVisible', false);

		return;
	}

	%max = %player.getMaxHealth();
	%currHP = %player.getHealth() + (%player.pseudoHealth + %player.tf2Overheal); //supports Item_Pills and TF2 medigun add-ons
					
	%dmgPercent = %player.getHealthLevel() - (%player.pseudoHealth + %player.tf2Overheal);
	%dmgPercent /= %player.getDatablock().maxDamage;

	if($HealthBar::Mode == 1)
		%hp = mClampF(%currHP, 0, %max + (%player.pseudoHealth + %player.tf2Overheal)) SPC %max;

	if(%val || %player.getDataBlock().isInvincible) //I means they are invincible, it's not always accurate
		%hp = "I";
	
	if(!%player.healthBarVisible)
	{
		%player.healthBarVisible = true;
		commandToClient(%client, 'setHealthBarVisible', true);
	}
	
	commandToClient(%client, 'setDamageLevel', %dmgPercent, %hp);
}

function GameConnection::sendVehicleHealthData(%client, %val)
{
	if(isObject(%player = %client.player))
		%player.sendVehicleHealthData(%val);
}

function ShapeBase::sendVehicleHealthDataAll(%obj, %val)
{
	%count = %obj.getMountedObjectCount();

	for(%i = 0; %i < %count; %i++)
	{
		%player = %obj.getMountedObject(%i);

		if(%player.getClassName() $= "Player")
			%player.sendVehicleHealthData();
	}
}

function Player::sendVehicleHealthData(%player, %val)
{
	if(!isObject(%client = %player.client) || !%client.hasHealthBarVehicle)
		return;

	if($HealthBar::MiniOnly && !isObject(%client.minigame))
	{
		%player.vehicleHealthBarVisible = false;

		commandToClient(%client, 'setVehicleDamageLevel', 0, 0);
		commandToClient(%client, 'setHealthBarVehicleVisible', false);

		return;
	}

	if(!$HealthBar::Enabled || !$HealthBar::Vehicles || %player.getState() $= "dead" || !isObject(%vehicle = %player.getObjectMount()))
	{
		%player.vehicleHealthBarVisible = false;

		commandToClient(%client, 'setVehicleDamageLevel', 0, 0);
		commandToClient(%client, 'setHealthBarVehicleVisible', false);

		return;
	}

	while(isObject(%vehicle.getObjectMount()))
	{
		//We want to get the base vehicle. Example: If they're in a tank turret, we want to get the health of the main vehicle part of the tank.
		%vehicle = %vehicle.getObjectMount();
	}

	if(%vehicle.isDisabled() || (%vehicle.getClassName() $= "Player" && %vehicle.getDataBlock().shapeFile $= "base/data/shapes/player/m.dts"))
	{
		%player.vehicleHealthBarVisible = false;

		commandToClient(%client, 'setVehicleDamageLevel', 0, 0);
		commandToClient(%client, 'setHealthBarVehicleVisible', false);

		return;
	}

	%max = %vehicle.getMaxHealth();
	%currHP = %vehicle.getHealth() + (%vehicle.pseudoHealth + %vehicle.tf2Overheal); //players can ride things like bots and horse players too, so let's also support those mods here
					
	%dmgPercent = %vehicle.getHealthLevel() - (%vehicle.pseudoHealth + %vehicle.tf2Overheal);
	%dmgPercent /= %vehicle.getDatablock().maxDamage;

	if($HealthBar::Mode == 1)
		%hp = mClampF(%currHP, 0, %max + (%vehicle.pseudoHealth + %vehicle.tf2Overheal)) SPC %max;

	if(%val || %vehicle.getDataBlock().isInvincible) //I means they are invincible, it's not always accurate
		%hp = "I";
	
	if(!%player.vehicleHealthBarVisible)
	{
		%player.vehicleHealthBarVisible = true;
		commandToClient(%client, 'setHealthBarVehicleVisible', true);
	}
	
	commandToClient(%client, 'setVehicleDamageLevel', %dmgPercent, %hp);
}